package xic.analysis;

import java.util.Set;

import xic.analysis.AvailableExpressions.ExprValue;
import xic.ir.IRNode;
import xic.ir.StructHashSet;


public class Delayedness extends DataFlowAnalysis {
	private AnticipatedExpressions antExpr;
	private EarliestExpressions earlExpr;

	public Delayedness(Set<CFGNode> nodes, CFGNode entry, 
			AnticipatedExpressions antExpr,
			EarliestExpressions earlExpr) {
		super(nodes);
		this.antExpr = antExpr;
		this.earlExpr = earlExpr;
		initFlowInfo();
		this.top = new ExprValue(((DelayednessFlowInfo) getFlowInfo(entry)).aneaIn);
		dir = Direction.FORWARD;
	}

	public class DelayednessFlowInfo extends FlowInfo {
		StructHashSet<IRNode> antLoc, aneaIn;

		public DelayednessFlowInfo(Object o) {
			CFGNode node = (CFGNode)o;
			antLoc = antExpr.eval(node);
			aneaIn = new StructHashSet<IRNode>(antExpr.in(node));
			aneaIn.retainAll(earlExpr.in(node));
			entry = new ExprValue();
			exit = new ExprValue();
		}
	};

	public StructHashSet<IRNode> in(CFGNode node) {
		DelayednessFlowInfo info = (DelayednessFlowInfo) flowInfo.get(node);
		return ((ExprValue) info.entry).exprs;
	}
	
	public StructHashSet<IRNode> out(CFGNode node) {
		DelayednessFlowInfo info = (DelayednessFlowInfo) flowInfo.get(node);
		return ((ExprValue) info.exit).exprs;
	}

	protected void F(FlowInfo f) {
		DelayednessFlowInfo fi = (DelayednessFlowInfo) f;

		StructHashSet<IRNode> entry = ((ExprValue) f.entry).exprs;
		StructHashSet<IRNode> exit = ((ExprValue) f.exit).exprs;
		
		entry.addAll(fi.aneaIn);

		exit.clear();
		exit.addAll(entry);
		exit.removeAll(fi.antLoc);
	}

	protected FlowInfo createFlowInfo(CFGNode node) {
		return new DelayednessFlowInfo(node);
	}
}
