package xic.analysis;

import java.util.Set;

import xic.ir.IRNode;
import xic.ir.StructHashSet;


public class EarliestExpressions extends AvailableExpressions {
	private AnticipatedExpressions ante;

	public EarliestExpressions(AnticipatedExpressions ante, Set<CFGNode> nodes) {
		super(nodes);
		dir = Direction.FORWARD;	
		this.ante = ante;
		top = new EarlyValue(allExprs);
	}

	public StructHashSet<IRNode> in(CFGNode node) {
		EarlyExprInfo info = (EarlyExprInfo)flowInfo.get(node);
		return ((EarlyValue) info.entry).exprs;
	}
	
	public StructHashSet<IRNode> out(CFGNode node) {
		EarlyExprInfo info = (EarlyExprInfo)flowInfo.get(node);
		return ((EarlyValue) info.exit).exprs;
	}

	protected void F(FlowInfo f) {
		
		EarlyExprInfo fi = (EarlyExprInfo) f;

		StructHashSet<IRNode> entry = ((EarlyValue) f.entry).exprs;
		StructHashSet<IRNode> exit = ((EarlyValue) f.exit).exprs;

		exit.clear();
		exit.addAll(entry);
		exit.removeAll(ante.in(fi.node));

		StructHashSet<IRNode> notTrans = computeKill(fi.kill, allExprs);
		exit.addAll(notTrans);
	}

	@Override
	protected FlowInfo createFlowInfo(CFGNode node) {
		return new EarlyExprInfo(node);
	}

	/* Dataflow-related information associated with each node */
	public class EarlyExprInfo extends AvailExprInfo {
		public EarlyExprInfo(Object o) {
			super(o);
			node = (CFGNode)o;
			entry = new EarlyValue();
			exit = new EarlyValue();			
		}
	}
	public class EarlyValue extends ExprValue {
		public void meet(Value v) {
			EarlyValue aevalue = (EarlyValue)v;
			exprs.addAll(aevalue.exprs);
		}

		//Initialize entry to allExprs
		public void setNullMeet() {
			exprs.addAll(allExprs);
		}
		
		public EarlyValue() {
			exprs = new StructHashSet<IRNode>();
		}

		public EarlyValue(StructHashSet<IRNode> exprs) {
			this.exprs = new StructHashSet<IRNode>(exprs);
	
		}
		public Value copy() {
			return new EarlyValue(exprs);
		}

	}
}
