package xic.analysis;

import java.util.HashSet;
import java.util.Set;

public class Postdominators extends DataFlowAnalysis {
	/// Value class used during postdominator analysis
	public static class PostdomValue implements Value {
		/* List of variable names */
		private Set<CFGNode> postdominators;

		public PostdomValue() {
			postdominators = new HashSet<CFGNode>();
		}

		public PostdomValue(Set<CFGNode> postDominators) {
			this.postdominators = new HashSet<CFGNode>(postDominators);
		}

		public Value copy() {
			return new PostdomValue(postdominators);
		}

		public void meet(Value v) {
			postdominators.retainAll(((PostdomValue) v).postdominators);
		}

		public String toString() {
			return postdominators.toString();
		}

		public int hashCode() {
			return postdominators.hashCode();
		}
		
		public Set<CFGNode> dominators() {
			return postdominators;
		}
		
		public boolean equals(Object obj) {
			if (!(obj instanceof PostdomValue))
				return false;
			return postdominators.equals(((PostdomValue) obj).postdominators);
		}

		public void setNullMeet() {
			postdominators.clear();
		}
	};

	/// Dataflow-related information associated with each node
	public static class PostDomFlowInfo extends FlowInfo {
		public CFGNode node;

		public PostDomFlowInfo(CFGNode node) {
			entry = new PostdomValue();
			exit = new PostdomValue();
			this.node = node;
		}
	};

	public Postdominators(Set<CFGNode> nodes) {
		super(nodes);

		dir = Direction.BACKWARD;
		top = new PostdomValue(nodes);
	}

	protected FlowInfo createFlowInfo(CFGNode node) {
		return new PostDomFlowInfo(node);
	}

	public Set<CFGNode> getPostdominators(CFGNode n) {
		return ((PostdomValue) ((PostDomFlowInfo) getFlowInfo(n)).exit).postdominators;
	}

	public void F(FlowInfo flowInfo) {
		PostDomFlowInfo f = (PostDomFlowInfo) flowInfo;
		Set<CFGNode> entry = ((PostdomValue) f.entry).postdominators;
		Set<CFGNode> exit = ((PostdomValue) f.exit).postdominators;
		
		exit.clear();
		exit.addAll(entry);
		exit.add(f.node);
	}
}
