package xic.ast;

import java.util.Arrays;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Compilation unit - contains the AST of a single source file.
 */
public class CompUnit extends ASTNode {
	private UseBlock useBlock;
	private DeclBlock globalStmtBlock;
	private String name;

	public CompUnit(Position pos, UseBlock useBlock, DeclBlock globalStmtBlock) {
		super(pos);
		this.useBlock = useBlock;
		this.globalStmtBlock = globalStmtBlock;
		name = pos.unit();
	}

	public String name() {
		return name;
	}

	public Iterable<VisualizableTreeNode> children() {
		return Arrays.asList(new VisualizableTreeNode[] {
			useBlock, globalStmtBlock
		});
	}
	
	public DeclBlock globalStmtBlock() {
		return globalStmtBlock;
	}
	
	public UseBlock useBlock() {
		return useBlock;
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	UseBlock useBlock = (UseBlock) v.visit(this, this.useBlock);
    	DeclBlock globalStmtBlock = (DeclBlock) v.visit(this, this.globalStmtBlock);

		if (useBlock != this.useBlock || globalStmtBlock != this.globalStmtBlock)
			return new CompUnit(position, useBlock, globalStmtBlock);

		return this;
	}

	public Copyable copy() {
		return new CompUnit(position, useBlock, globalStmtBlock);
	}
    
	public String label() {
		return "compUnit";
	}
}
