package xic.ast;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Stored a list of declarations (either variable, function- or class declarations)
 */
public class DeclBlock extends ASTNode {
	private List<Decl> decls;
	private Map<String, TypeNode> symbolTable;

	public DeclBlock() {
		super(null);
		decls = new ArrayList<Decl>();
	}
	
	public DeclBlock(Position pos, List<Decl> decls, Map<String, TypeNode> symbolTable) {
		super(pos);
		this.decls = decls;
		this.symbolTable = symbolTable;
	}

	public DeclBlock(Decl decl) {
		super(decl.position());
		decls = new ArrayList<Decl>();
		decls.add(decl);
	}

	public void appendDecl(Decl decl) {
		decls.add(decl);
		extendPositionTo(decl.position());
	}

	public List<Decl> decls() {
		return decls;
	}
	
	public Map<String, TypeNode> getSymbolTable() {
		return symbolTable;
	}
	
	public DeclBlock setSymbolTable(Map<String, TypeNode> symbolTable) {
		if (symbolTable == this.symbolTable)
			return this;
		DeclBlock b = (DeclBlock) copy();
		b.symbolTable = symbolTable;
		return b;
	}
	

    public ASTNode visitChildren(NodeVisitor v) {
    	boolean modified = false;

    	ArrayList<Decl> results = new ArrayList<Decl>();
    	for (Decl decl : decls) {
    		Decl newDecl = (Decl) v.visit(this, decl);
    		if (newDecl != decl)
    			modified = true;
    		results.add(newDecl);
    	}

		if (modified)
			return new DeclBlock(position, results, symbolTable);

		return this;
	}

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>(
				decls.size());
		result.addAll(decls);
		return result;
	}

	public String label() {
		return "declBlock";
	}

	public Copyable copy() {
		return new DeclBlock(position, new ArrayList<Decl>(decls), symbolTable);
	}
}