package xic.ast;

import java.util.ArrayList;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Local variable declaration. Initialization is optional.
 */
public class LocalVarDecl extends Stmt {
	private TupleDecl tupleDecl;
	private Expr init;

	public LocalVarDecl(Position pos, TupleDecl tupleDecl, Expr init) {
		super(pos);
		this.tupleDecl = tupleDecl;
		this.init = init;		
	}

	public LocalVarDecl(Position pos, TupleDecl tupleDecl) {
		super(pos);
		this.tupleDecl = tupleDecl;
	}

	/**
	 * Copy constructor, makes a shallow copy while
	 * providing new position information.
	 */
	public LocalVarDecl(Position pos, LocalVarDecl block) {
		super(pos);
		this.tupleDecl = block.tupleDecl();
		this.init = block.init();
	}

	public boolean hasInit() {
		return init != null;
	}
	
	public Expr init() {
		return init;
	}

	public TupleDecl tupleDecl() {
		return tupleDecl;
	}

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>();
		result.add(tupleDecl);
		if (hasInit())
			result.add(init);
		return result;
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	TupleDecl tupleDecl = (TupleDecl) v.visit(this, this.tupleDecl);
    	Expr init = (Expr) v.visit(this, this.init);

		if (tupleDecl != this.tupleDecl || init != this.init)
			return new LocalVarDecl(position, tupleDecl, init);

		return this;
	}

    public Copyable copy() {
		return new LocalVarDecl(position, tupleDecl, init);
    }

	public String label() {
		return "localVarDecl";
	}
}
