package xic.ast;

import java.util.ArrayList;
import java.util.List;

import xic.visitors.NodeVisitor;

import edu.cornell.cs.cs4120.xi.Position;
import edu.cornell.cs.cs4120.util.VisualizableTreeNode;

/**
 * Variable declaration - either global or as a class member. For
 * local variable declarations, see <code>LocalVarDecl</code>
 */
public class VarDecl extends Decl {
	private TupleDecl tupleDecl;
	private TypeNode type;
	private boolean imported;
	private Expr init;

	public VarDecl(Position pos, TypeNode type, TupleDecl tupleDecl, Expr init, boolean imported) {
		super(pos);
		this.tupleDecl = tupleDecl;
		this.imported = imported;
		this.init = init;
		this.type = type;
	}

	public VarDecl(Position pos, TypeNode type, TupleDecl tupleDecl,boolean imported) {
		super(pos);
		this.tupleDecl = tupleDecl;
		this.imported = imported;
		this.type = type;
	}

	public TupleDecl tupleDecl() {
		return tupleDecl;
	}
	
	public boolean isImported() {
		return imported;
	}
	
	public TypeNode type() {
		return type;
	}
	
	public Expr init() {
		return init;
	}
	
	public boolean hasInit() {
		return init != null;
	}

	public Iterable<VisualizableTreeNode> children() {
		ArrayList<VisualizableTreeNode> result = new ArrayList<VisualizableTreeNode>();
		result.add(tupleDecl);
		if (hasInit())
			result.add(init);
		return result;
	}
	
	public List<String> getNames() {
		return tupleDecl.getNames();
	}

    public ASTNode visitChildren(NodeVisitor v) {
    	TupleDecl tupleDecl = (TupleDecl) v.visit(this, this.tupleDecl);
    	Expr init = (Expr) v.visit(this, this.init);

		if (tupleDecl != this.tupleDecl || init != this.init)
			return new VarDecl(position, type, tupleDecl, init, imported);

		return this;
	}

    public Copyable copy() {
		return new VarDecl(position, type, tupleDecl, init, imported);
    }

	public String label() {
		return "varDecl";
	}
}
