package xic.codegen.matchers;

import xic.codegen.Instr;
import xic.codegen.InstrList;
import xic.codegen.Matcher;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.ir.IRConst;
import xic.ir.IRNode;
import xic.ir.NameGenerator;

public class ConstMatcher implements Matcher {
	public static class ConstTile implements Tile {
		private long value;

		public ConstTile(long value) {
			this.value = value;
		}
		
		public int estimateCost() {
			return 1;
		}
		
		public boolean is32BitOrLess() {
			return value <= (long) Integer.MAX_VALUE && value >= (long) Integer.MIN_VALUE;
		}

		public String translate(InstrList il, NameGenerator ng) {
			String tempName = ng.nextTemp();
			il.add(new Instr("mov", new Instr.Op(Instr.Op.Type.REG,
					tempName), new Instr.Op(value)));
			return tempName;
		}

		public long value() {
			return value;
		}
	};
	
	public Tile match(IRNode node, Tiler tiler) {
		if (!(node instanceof IRConst))
			return null;
		return new ConstTile(((IRConst) node).value());
	}
}
