package xic.codegen.matchers;

import xic.Configuration;
import xic.codegen.AbstractTile;
import xic.codegen.AsmDirective;
import xic.codegen.Instr;
import xic.codegen.InstrList;
import xic.codegen.LabelInstr;
import xic.codegen.Matcher;
import xic.codegen.RegAlloc;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.ir.IRFuncDecl;
import xic.ir.IRNode;
import xic.ir.NameGenerator;

public class FuncDeclMatcher implements Matcher {
	protected static class FuncDeclTile extends AbstractTile {
		private String funcName;
		private boolean isProcedure = false;
		public FuncDeclTile(String funcName, Tile body, boolean isProcedure) {
			this.funcName = funcName;
			this.isProcedure = isProcedure;
			children.add(body);
		}

		public int localCost() {
			return 0;
		}

		public String translate(InstrList il, NameGenerator ng) {
			il.add(new AsmDirective(""));
			// Don't export symbols for internal functions
			if (funcName.startsWith("_"))
				il.add(new AsmDirective(".globl " + funcName));
			il.add(new LabelInstr(funcName));
			if (Configuration.IP_RELATIVE_ADDRESSING)
				il.add(new Instr("declare", new Instr.Op(Instr.Op.Type.REG, Configuration.IP_REGISTER)));
			children.get(0).translate(il, ng);
			if (!isProcedure) {
				il.add(new Instr("mov", 
					new Instr.Op(Instr.Op.Type.REG, Configuration.RETVAL_REGISTER),
					new Instr.Op(Instr.Op.Type.REG, RegAlloc.RV_NAME)
				));
			}
			return null;
		}
	};

	public Tile match(IRNode node, Tiler tiler) {
		if (!(node instanceof IRFuncDecl))
			return null;

		IRFuncDecl func = (IRFuncDecl) node;
		return new FuncDeclTile(func.name(), tiler.tile(func.body()), func.isProcedure());
	}
}
