package xic.codegen.matchers;

import xic.Configuration;
import xic.codegen.Instr;
import xic.codegen.Instr.Op;
import xic.codegen.InstrList;
import xic.codegen.Matcher;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.ir.IRName;
import xic.ir.IRNode;
import xic.ir.NameGenerator;

public class NameMatcher implements Matcher {
	public static class NameTile implements Tile {
		private String name;

		public NameTile(String name) {
			this.name = name;
		}
		
		public int estimateCost() {
			return 1;
		}

		public String translate(InstrList il, NameGenerator ng) {
			String tempName = ng.nextTemp();
			Instr.Op nameOp = new Instr.Op(Op.Type.MEM);
			nameOp.setImmediateName(name);
			if (Configuration.IP_RELATIVE_ADDRESSING)
				nameOp.setBase(Configuration.IP_REGISTER);
			il.add(new Instr("lea", new Instr.Op(Instr.Op.Type.REG, tempName), nameOp));
			return tempName;
		}

		public String name() {
			return name;
		}
	};
	
	public Tile match(IRNode node, Tiler tiler) {
		if (!(node instanceof IRName))
			return null;

		return new NameTile(((IRName) node).name());
	}
}
