package xic.codegen.matchers;

import xic.Configuration;
import xic.Configuration.Target;
import xic.codegen.AbstractTile;
import xic.codegen.AsmDirective;
import xic.codegen.InstrList;
import xic.codegen.LabelInstr;
import xic.codegen.Matcher;
import xic.codegen.Tile;
import xic.codegen.Tiler;
import xic.ir.IRNode;
import xic.ir.IRZData;
import xic.ir.NameGenerator;

public class ZDataMatcher implements Matcher {
	protected static class ZDataTile extends AbstractTile {
		private int size;
		private String name;

		public ZDataTile(String name, int size) {
			this.name = name;
			this.size = size;
		}

		public int localCost() {
			return 0;
		}

		public String translate(InstrList il, NameGenerator ng) {
			boolean osx = Configuration.TARGET == Target.DARWIN;
			il.add(new AsmDirective(osx ? "\t.data" : "\t.bss"));
			il.add(new AsmDirective("\t.align "+ (osx ? Configuration.LOG_WORD_SIZE : Configuration.WORD_SIZE)));
			if (name.startsWith("_"))
				il.add(new AsmDirective(".globl "+name));
			if (osx) {
				il.add(new AsmDirective("\t.lcomm " + name + ", "+size));
			} else {
				il.add(new LabelInstr(name));
				il.add(new AsmDirective("\t.zero "+size));
			}
			il.add(new AsmDirective("\t.text"));
			il.add(new AsmDirective(""));

			return null;
		}
	};

	public Tile match(IRNode node, Tiler tiler) {
		if (node instanceof IRZData) {
			IRZData data = (IRZData) node;
			return new ZDataTile(data.name(), data.size());
		}
		
		return null;
	}
}
