package xic.ir;

import java.util.Set;

import xic.ast.ASTNode;
import xic.ast.Copyable;
import xic.visitors.NodeVisitor;

/**
 * Fake declaration node, which is used to "anchor" registers
 * that are used to pass arguments to functions
 */
public class IRDecl extends IRStmt {
	private String name;
	
	public IRDecl(String name) {
		this.name = name;
	}

	public String label() {
		return "DECL_REG(" + name + ")";
	}

	public String name() {
		return name;
	}

	public boolean containsCalls() {
		return false;
	}
	
	public boolean containsMemReads() {
		return false;
	}

	public boolean containsMemWrites() {
		return false;
	}

	public ASTNode visitChildren(NodeVisitor v) {
		return this;
	}

	public void findReadTemps(Set<String> tempNames) {
	}

	public void findWrittenTemps(Set<String> tempNames) {
		tempNames.add(name);
	}

	public int computeMaximumCallArguments() {
		return 0;
	}

	public int computeMaximumCallResults() {
		return 0;		
	}

	public int nodeCount() {
		return 1;
	}

	public Copyable copy() {
		return new IRDecl(name);
	}
	
	public Copyable deepCopy() {
		return new IRDecl(name);
	}
	
	public boolean equalsTree(Object object) {
		if (!(object instanceof IRDecl))
			return false;
		return (((IRDecl) object).name.equals(name));
	}
	
	public int treeHashCode() {
		return name.hashCode();
	}
}