package xic.ir;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public final class StructHashSet<V> implements Set<V> {
	public final static class StructWrapper {
		IRNode value;

		public StructWrapper(IRNode e) {
			this.value = e;
		}

		public IRNode value(){
			return value;
		}
		public boolean equals(Object obj) {
			return value.equalsTree(((StructWrapper) obj).value);
		}
		
		public String toString() {
			return value.toString();
		}

		public int hashCode() {
			return value.treeHashCode();
		}
	};
	private Set<StructWrapper> wrappedSet;
	
	public StructHashSet() {
		super();
		wrappedSet = new HashSet<StructWrapper>();
	}

	public StructHashSet(StructHashSet<V> v) {
		super();
		wrappedSet = new HashSet<StructWrapper>();
		addAll(v);
	}

	public boolean add(V e) {
    	return wrappedSet.add(new StructWrapper((IRNode) e));
	}

	public boolean addAll(Collection<? extends V> c) {
		boolean changed = false;
		for(Object expr : c )
			changed |= wrappedSet.add(new StructWrapper((IRNode) expr));
		return changed;
	}

	@Override
	public void clear() {
		wrappedSet.clear();
	}

	@Override
	public boolean contains(Object o) {
    	return wrappedSet.contains(new StructWrapper((IRNode) o));
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		for(Object expr : c )
			if(!wrappedSet.contains(new StructWrapper((IRNode) expr)))
				return false;
		return true;
	}

	public boolean isEmpty() {
		return wrappedSet.isEmpty();
	}

	@SuppressWarnings("unchecked")
	public Iterator<V> iterator() {
		final Iterator<StructWrapper> it = wrappedSet.iterator();
		return new Iterator<V>() {
			public boolean hasNext() {
				return it.hasNext();
			}

			public V next() {
				return (V) it.next().value;
			}

			public void remove() {
				it.remove();
			}
		};
	}

	public boolean remove(Object o) {
    	return wrappedSet.remove(new StructWrapper((IRNode) o));
	}

	public boolean removeAll(Collection<?> c) {
		boolean changed = false;
		for(Object expr : c )
			changed |= wrappedSet.remove(new StructWrapper((IRNode) expr));
		return changed;
	}

	@SuppressWarnings("unchecked")
	public boolean retainAll(Collection<?> c) {
		if(c instanceof StructHashSet<?>)
			return retainAll((StructHashSet<IRNode>)c);
		else
			throw new UnsupportedOperationException("can only operate on StructHashSets");
	}
	
	public boolean retainAll(StructHashSet<IRNode> c) {
		boolean changed = false;
		Set<StructWrapper> wscopy = new HashSet<StructWrapper>(wrappedSet);
		for(StructWrapper w : wscopy )
			if(!c.contains(w.value))
				changed |= wrappedSet.remove(w);
		
		return changed;
	}
	
	public static StructHashSet<IRNode> diff(StructHashSet<IRNode> a, StructHashSet<IRNode> b) {
		StructHashSet<IRNode> both = new StructHashSet<IRNode>(a);
		StructHashSet<IRNode> acopy = new StructHashSet<IRNode>(a);
		StructHashSet<IRNode> bcopy = new StructHashSet<IRNode>(b);

		both.retainAll(b);
		acopy.removeAll(both);
		bcopy.removeAll(both);
		
		StructHashSet<IRNode> diff = new StructHashSet<IRNode>();
		diff.addAll(acopy);
		diff.addAll(bcopy);
		return diff;
	}

	public int size() {
		return wrappedSet.size();
	}

	public Object[] toArray() {
		return wrappedSet.toArray();
	}

	public <T> T[] toArray(T[] a) {
		return wrappedSet.toArray(a);
	}
	
	public int hashCode() {
		return wrappedSet.hashCode();
	}
	
	public boolean equals(StructHashSet<V> shs) {
		return wrappedSet.equals(shs.wrappedSet);
	}
	public String toString(){
		String str = "<\n";
		for(StructWrapper expr : wrappedSet)
			str+="\t["+ expr.value.toString()+"]\n";
		str+=">";
		return str; 
	}
}
