package xic.lexer;

import xic.PositionImpl;
import xic.Trackable;
import edu.cornell.cs.cs4120.xi.Position;

import static xic.parser.TokenType.*;

/**
 * Basic token implementation - stores position
 * information as well as its source code representation.
 */
public class BaseToken implements Trackable {
	private Position position;
	private String value;
	private int type;

	public BaseToken(int type, String value, String unit, int line,
			int column, int length) {
		this.position = new PositionImpl(unit, line, line, column, column
				+ length-1);
		this.type = type;
		this.value = value;
	}

	public BaseToken(int type,Position position, String value) {
		this.type = type;
		this.position = position;
		this.value = value;
	}

	public Position position() {
		return position;
	}

	public String value() {
		return value;
	}

	public int type() {
		return type;
	}

	public boolean equals(Object obj) {
		if (obj.getClass() != BaseToken.class)
			return false;
		BaseToken other = (BaseToken) obj;
		return other.position.equals(position) && this.type() == other.type()
				&& other.value.equals(value);

	}

	public int hashCode() {
		int result = 17;
		result = 37*result + position.hashCode();
		result = 37*result + this.type();
		result = 37*result + value.hashCode();
		return result;
	}

	/**
	 * Return a string representation of the symbol type
	 */
	public String symString() {
		switch (this.type()) {
			case SOURCE_IDENTIFIER: return "SOURCE_IDENTIFIER";
			case HEADER_IDENTIFIER: return "HEADER_IDENTIFIER";
		    case DIVIDE: return "DIVIDE"; 
			case STRING_LITERAL: return "STRING_LITERAL"; 
			case OPEN_PAREN: return "OPEN_PAREN"; 
			case NOT_EQUAL: return "NOT_EQUAL"; 
			case INT: return "INT"; 
			case MINUS: return "MINUS"; 
			case SEMICOLON: return "SEMICOLON"; 
			case CHARACTER_LITERAL: return "CHARACTER_LITERAL"; 
			case NOT: return "NOT"; 
			case AND: return "AND"; 
			case LT: return "LT"; 
			case CLOSE_PAREN: return "CLOSE_PAREN"; 
			case USE: return "USE"; 
			case OR: return "OR"; 
			case COMMA: return "COMMA"; 
			case BOOL: return "BOOL"; 
			case GEQ: return "GEQ"; 
			case PLUS: return "PLUS"; 
			case IF: return "IF"; 
			case EOF: return "EOF"; 
			case RETURN: return "RETURN"; 
			case EQUAL: return "EQUAL"; 
			case TRUE: return "TRUE"; 
			case error: return "error"; 
			case MODULO: return "MODULO"; 
			case OPEN_BRACE: return "OPEN_BRACE"; 
			case CLOSE_BRACKET: return "CLOSE_BRACKET"; 
			case BREAK: return "BREAK"; 
			case OPEN_BRACKET: return "OPEN_BRACKET"; 
			case TIMES: return "TIMES"; 
			case COLON: return "COLON"; 
			case ELSE: return "ELSE"; 
			case WHILE: return "WHILE"; 
			case LEQ: return "LEQ"; 
			case UNDERSCORE: return "UNDERSCORE"; 
			case CLOSE_BRACE: return "CLOSE_BRACE"; 
			case GETS: return "GETS"; 
			case FALSE: return "FALSE"; 
			case GT: return "GT"; 
			case LENGTH: return "LENGTH"; 
			case INTEGER_LITERAL: return "INTEGER_LITERAL"; 
			case IDENTIFIER: return "IDENTIFIER"; 
			default: return "invalid";
		}
	}
	
	public String toString() {
		return "Token[position=" + position + ", symbol=" + symString() + ", value="
				+ value + "]";
	}
}
