package xic.visitors;

import java.util.ArrayList;
import java.util.List;

import xic.ast.ASTNode;


/**
 * Extends the node visitor with functionality to ignore
 * classes part of a pre-defined list.
 */
public class SelectiveVisitor extends NodeVisitor {
	protected List<Class<? extends ASTNode>> ignoredTypes;

	public SelectiveVisitor() {
		ignoredTypes = new ArrayList<Class<? extends ASTNode>>();
	}

	protected SelectiveVisitor(List<Class<? extends ASTNode>> ignoredTypes) {
		this.ignoredTypes = ignoredTypes;
	}

	public List<Class<? extends ASTNode>> getIgnoredTypes() {
		return ignoredTypes;
	}
	
	/**
	 * Add an AST node type to the ignore list
	 */
	public void addIgnore(Class<? extends ASTNode> theClass) {
		ignoredTypes.add(theClass);
	}

	/**
	 * Remove an AST node type from the ignore list
	 */
	public void removeIgnore(Class<? extends ASTNode> theClass) {
		ignoredTypes.remove(theClass);
	}

	protected ASTNode override(ASTNode parent, ASTNode n) {
		for (Class<? extends ASTNode> c : ignoredTypes) {
			if (c.isAssignableFrom(n.getClass()))
				return n;
		}
		return null;
	}
}
