package xic;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestName;

import xic.ast.ASTNode;
import xic.ast.CompUnit;
import xic.parser.IotaParser;
import xic.parser.ParserImpl.Language;
import xic.visitors.TypeChecker;
import edu.cornell.cs.cs4120.xi.CompilationException;

public class TypeCheckerErrorTest {
	@Rule public TestName name = new TestName();

	private void testError(int count, String input) throws CompilationException {
		ASTNode parsedAST = IotaParser.getAST(input);
		ErrorSink errorSink = new ErrorSink();
		
		ArrayList<String> searchPath = new ArrayList<String>();
		searchPath.add(ExecutableTest.getIotaIncludeDirectory());

		new TypeChecker(Language.IOTA_09, errorSink, searchPath).run((CompUnit) parsedAST);

		/* Uncomment to look at error messages */
		/*
		for (CompilationException e : errorSink.getErrors()) {
			System.out.println("Error message in failure testcase '"
					+ name.getMethodName() + "': \"" + e.getMessage() + "\"");
		}*/
		assertEquals(count, errorSink.getErrors().size());
	}

	@Test
	public void test1() {
		// Incorrect types
		testError(1,
			"test() = {\n" 
			+ "  (a:bool, b:int) = (1, true);"
			+ "}"
		);
	}

	@Test
	public void test2() {
		// Can't override names
		testError(1,
			"test() = {\n" 
			+ "  (a:bool, (a:bool, _)) = (false, (true, 3));"
			+ "}"
		);
	}

	@Test
	public void test3() {
		// Missing return value
		testError(1,
			"test():int = {\n" 
			+ "  return;"
			+ "}"
		);
	}

	@Test
	public void test4() {
		// Incorrect return value type
		testError(1,
			"test():int = {\n" 
			+ "  return false;"
			+ "}"
		);
	}

	@Test
	public void test5() {
		// Misplaced break statement
		testError(1,
			"test() = {\n" 
			+ "  break;"
			+ "}"
		);
	}
	
	@Test
	public void test6() {
		// Declare an existing name 
		testError(1,
				"test() = {\n" 
				+ "  x:int = 2;"
				+ "  x:bool=true;"
				+ "}"
			);
	}

	@Test
	public void test7() {
		// type mismatch in length expression 
		testError(1,
				"test() = {\n" 
				+ "  x:bool=true;"
				+ "  y:bool=length x;"
				+ "}"
			);
	}
	
	@Test
	public void test8() {
		// invalid unary operations
		testError(4,
				"test() = {\n" 
				+ "  x:bool=true;"
				+ "  y:int= 1;"
				+ "  a:bool=-x;"
				+ "  b:int=!y;"
				+ "  c:(bool, bool)=!(x,x);"
				+ "  d:(int, int)=-(y,y);"
				+ "}"
			);
	}
	
	@Test
	public void test9() {
		// invalid binary operations 
		testError(13,
				"test() = {\n" 
				+ "  b1:bool=true;"
				+ "  b2:bool=false;"
				+ "  b3:bool;"
				+ "  x:int= 2;"
				+ "  y:int= 1;"
				+ "  z:int;"
				+ "  z=b1+x;"
				+ "  z=b1-b2;"
				+ "  z=x*b1;"
				+ "  z=b1/b2;"
				+ "  z=b1%x;"
				+ "  b3=(b1==x);"
				+ "  b3=(x!=b2);"
				+ "  b3=b1<b2;"
				+ "  b3=b1<=x;"
				+ "  b3=x>b1;"
				+ "  b3=x>=b1;"
				+ "  b3=x&y;"
				+ "  b3=x|b1;"
				+ "}"
			);
	}
	
	@Test
	public void test10() {
		// name conflict 
		testError(1,
			"test() = {\n"
			+ "test:int[] = (1,2,3,4);\n"
			+ "}"
		);
	}

	@Test
	public void test11() {
		// element type does not match when compare two arrays 
		testError(1,
			"test() = {\n"
			+ "x:int[];\n"
			+ "y:bool[];\n"
			+ "z:bool=(x==y);\n"
			+ "}"
		);
	}
	@Test
	public void test12() {
		// element type does not match 
		testError(1,
			"test() = {\n"
			+ "test1:int[];\n"
			+ "test1 = (2,3,true,5,4);\n"
			+ "}"
		);
	}
	@Test
	public void test13() {
		// element type does not match 
		testError(1,
			"test() = {\n"
			+ "test1:int[];\n"
			+ "test1 3 = true;\n"
			+ "}"
		);
	}
	@Test
	public void test14() {
		// Calling a function as if it were a procedure
		testError(1,
				"pepsi():int[] = {" +
				"    return \"Pepsi\";" +
				"}" +
				"illegalFunc() = {" +
				"    pepsi();" +
				"}"
				);
	}

	@Test
	public void test15() {
		// break after while 
		testError(1,
				"test() = {\n"
				+ "while(true)\n"
				+ "		return;\n"
				+ "break;\n" +
				"}"			
				);
	}


	@Test
	public void test16() {
		// A is not a subtype of any array
		testError(2,
				"test() = {" +
				"  a:(int, bool);" +
				"  b:int = a 0;" +
				"  c:int = a 1;" +
				"}"
		);
	}

	@Test
	public void test17() {
		// Invalid array indexing
		testError(5,
				"test() = {" +
				"  a:int[];" +
				"  a a = 1;" +
				"  a \"Hello\" = 1;" +
				"  a test = 1;" +
				"  a true = 1;" +
				"  a (1,2) = 1;" +
				"}"
		);
	}

	@Test
	public void test18() {
		// tuple comparison with incompatible types
		testError(1,
			"test() = {\n"
			+ "x:(bool, int);\n"
			+ "y:(int, bool);\n"
			+ "z:bool=(x==y);\n"
			+ "}"
		);
	}

	@Test
	public void test19() {
		// length(unit) is not permitted
		testError(1,
			"a() = { }"
			+ "test() = {\n"
			+ "  x:int=length(a());\n"
			+ "}"
		);
	}

	@Test
	public void test20() {
		// Returning a value in a procedure
		testError(1,
			"test() = {\n" 
			+ "  return false;"
			+ "}"
		);
	}

	@Test
	public void test22() {
		// Variable goes out of scope after a block
		testError(1,
			"test():int = {\n" 
				+ "  if (true) {"
				+ "  	a:int = 1;"
				+ "  }"
				+ "  return a;"
			+ "}"
		);
	}

	@Test
	public void test23() {
		// Ensure that the symbol table complains when importing
		// an interface twice
		testError(1,
			"use io;" + 
			"use io;" 
		
		);
	}

	@Test
	public void test24() {
		// Variable goes out of scope even if there is no block
		testError(1,
			"test():int = {\n" 
				+ "  if (true)"
				+ "  	a:int = 1;"
				+ "  return a;"
			+ "}"
		);
	}

	@Test
	public void test25() {
		//ArrayDecl length must be a subtype of int
		testError(1,
			"test() = {\n"
			+ "  a:int[true] = (1,2,3);"
			+ "}"
		);
	}

	@Test
	public void test26() {
		// length exprs are not permitted in function parameters
		testError(1,
			"test(a:int[1]) = {\n"
			+ "}"
		);
	}
	@Test
	public void test27() {
		// Assignment statements of type: 
		// array(i) = array(i);
		testError(1,
				 "h(a:int[]):int[] = {"
				+"    return a;"
				+"}"
				+"fun(x:int) = {"
				+"    h(x) = h(x);"
				+"    return;"
				+"}"
				);
	}	
	
	@Test
	public void test28() {
		// Assignment statements of type: 
		// tuple(i) = a;
		String source =
			"fun(x:int) = {" 
			+"	y:(int,(int,int)) = (1,(2,3));"
			+"  y 0 = 2;"
			+"  return;"
			+"}";
		
		testError(1,source);
	}
	
	@Test
	public void invalidLValue() {
		testError(1,
				"fun(x:int) = {" 
				+"  1 = 2;"
				+"  return;"
				+"}"
				);
	}	
	
	
	@Test
	public void test29() {
		// Spec update: cannot initialize global variables with computation
		testError(2,
			  "i:int = 1;" +
			  "j:int = i;" +
			  "k:int = 1+2;"
		);
	}

	@Test
	public void test30() {
		// Spec update: cannot have global variables with an inappropriate initialization
		testError(2,
			  "i:int = false;" +
			  "k:bool = 1;"
		);
	}
	
	@Test
	public void test31() {
		// Spec update: cannot have global array declarations with non-trivial length expressions 
		testError(1,
			  "i:int = 1;" +
			  "j:int[i+1];"
		);
	}

	@Test
	public void test32() {
		// Spec update: cannot have global array declarations with invalid orderings of length expressions 
		testError(1,
			  "j:int[][2];"
		);
	}
	
	@Test
	public void test33() {
		/* We support neither covariance nor contravariance for array types */
		testError(1, "class Point {};" +
		"class ColoredPoint extends Point { }"+
		"a() = {"+
		"    array1: ColoredPoint[5];"+
		"    array2 : Point[];"+
		"    array2 = array1;"+
		"	}");
	}
	
	
	@Test
	public void test34() {
		/* Cannot do assignements to non-homogeneous tuples */
		testError(1,
		"a() = {"+
		"   b:(int, (int, int));" +
		"   b 1 = 0;" +
		"}");
	}
	
	
	@Test
	public void test35() {
		/* Invalid assignments */
		testError(1,
		"a() = {"+
		"   a:int = null;" +
		"}");
	}

	@Test
	public void test36() {
		/* Can't use 'this' in a function */
		testError(1,
		"a() :int = {"+
		"   if (this == null)" +
		"       return 1;" +
		"   return 0;" +
		"}");
	}
}
